% conservativeReachTubeVisualize: generate a pretty plot
%
% This script runs conservativeReachTube(), and then generates a pretty
% visualization which shades in the target and inevitable collision sets
% and shows a sample trajectory which is part of the computed reach tube,
% but is not actually part of the backward minimal sampled data reach tube.

% Copyright 2011 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2009/11/28
% Subversion tags for version control purposes.
% $Date: 2011-11-29 23:00:42 -0800 (Tue, 29 Nov 2011) $
% $Id: conservativeReachTubeVisualize.m 653 2011-11-30 07:00:42Z mitchell $

  % Generate the estimated reach tube.
  max_steps = 3;
  % Figure in the paper was generated with veryHigh, but that takes 5-8
  % times longer.
  %accuracy = 'veryHigh';
  accuracy = 'medium';
  dx = 0.025;
  [ g, inev ] = conservativeReachTube(max_steps, accuracy, dx);
  
  % Get rid of the figure generated during computation.
  close;
  
  % Convert the inevitable sets over various horizons into a single stepped
  % function.  Regions outside the reach tube will be -1, in the target set
  % will be 0, and regions in the reach tube for step n will have value n.
  % We need to work our way backwards through the set of reach tubes
  % because each reach tube at step n contains the tube at step (n-1).
  summary = -ones(g.shape);
  for i = max_steps + 1 : -1 : 1;
    summary(inev{i} <= 0) = i-1;
  end
  
  % Create the plot.
  figure;
  contourf(g.xs{:}, summary, 0 : max_steps);
  title(sprintf('Reach Tube Estimate \\delta = 1, N = %d', max_steps))
  grid on;
  axis equal;
  axis(g.axis);
  
  % Make the color scheme more amenable to B&W printing.
  grayscale = gray(max_steps + 4);
  colormap(grayscale(2:end-2, :));
  
  % Now demonstrate a safe trajectory which lies within the estimated reach tube.
  hold on;
  xs = [ -0.5; +0.5; -0.5 ];
  ys = [ +1.5; +0.5; -0.5 ];
  plot(xs, ys, 'b-o', 'LineWidth', 2);
  